/* wptClipDecryptDlg.cpp - Clipboard decryption
 *	Copyright (C) 2000-2007 Timo Schulz
 *	Copyright (C) 2005 g10 Code GmbH
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <windows.h>
#include <assert.h>
#include <time.h>

#include "wptTypes.h"
#include "wptW32API.h"
#include "wptAgent.h"
#include "wptNLS.h"
#include "wptGPG.h"
#include "wptVersion.h"
#include "wptErrors.h"
#include "wptCommonCtl.h"
#include "wptContext.h"
#include "wptDlgs.h"
#include "wptKeylist.h"
#include "wptUTF8.h"
#include "resource.h"
#include "StringBuffer.h"

bool is_seckey_available (gpgme_recipient_t rset);
char* get_pka_status (gpgme_signature_t sig);


/* Return the primary user-ID of the key with the keyid @keyid. 
   Caller must free string. */
char*
get_key_userid (const char *keyid)
{
    winpt_key_s key;
    StringBuffer p;
    const char *userid;
    char *uid;

    p = "\n    ";   
    memset (&key, 0, sizeof (key));
    if (winpt_get_pubkey (keyid, &key))
	userid = (_("user ID not found"));
    else
	userid = key.ext->uids->uid;
    p = p + "\"" + userid + "\"";

    winpt_release_pubkey (&key);
    uid = p.getBufferCopy ();
    return uid;
}


/* Decrypt the clipboard contents and on success
   replace the data with the plaintext.
   Return value: 0 on success. */
gpgme_error_t
gpgme_op_clip_decrypt (gpgme_ctx_t ctx)
{
    gpgme_error_t err;
    gpgme_data_t ciph = NULL;
    gpgme_data_t plain = NULL;
    int is_utf8=0;
    
    err = gpg_data_utf8_new_from_clipboard (&ciph, 0, &is_utf8);
    if (err)
	return err;

    err = gpgme_data_new (&plain);
    if (err) {
	gpgme_data_release (ciph);
	return err;
    }

    err = gpgme_op_decrypt_verify (ctx, ciph, plain);
    if (is_utf8)
	gpg_data_release_utf8_to_clipboard (plain);
    else
	gpg_data_release_to_clipboard (plain, 0);
    gpgme_data_release (ciph);
    return err;
}


/* Return humand readable ownertrust description for verification info. */
const char*
verify_get_key_ownertrust (gpgme_validity_t key_ot, int *novalid)
{
    const char *s;

    if (novalid)
	*novalid = 0; /* reset */
    if (key_ot == GPGME_VALIDITY_FULL ||
	key_ot == GPGME_VALIDITY_ULTIMATE)
	s = _("Signature status: created with a fully trusted key");	
    else if (key_ot == GPGME_VALIDITY_MARGINAL)    
	s = _("Signature status: created with a marginal trusted key");
    else if (key_ot == GPGME_VALIDITY_NEVER) {
	if (novalid) 
	    *novalid = 1;
	s =  _("Signature status: created with an UNTRUSTED key");	
    }
    else
	s = _("Signature status: created with an undefined trusted key");
    return s;
}


/* Return a signature specific header and footer for the clipboard. */
void
verify_get_clip_info (gpgme_signature_t sig, char **r_header, char **r_footer)
{
    struct winpt_key_s pk;
    const char *head = _("*** PGP SIGNATURE VERIFICATION ***\r\n"
		         "*** Signature made:    %s\r\n"
		         "*** Signature verfied: %s\r\n"
		         "*** %s\r\n"
		         "*** Signature result:  %s\r\n"
		         "*** Signer: %s (0x%s)\r\n"
		         "*** BEGIN PGP DECRYPTED TEXT ***\r\n");
    const char *foot = _("\r\n*** END PGP DECRYPTED TEXT ***");
    const char *stat, *ot, *uid, *keyid;
    char made[128], ver[128];
    char *p;

    if (winpt_get_pubkey (sig->fpr, &pk)) {
	uid = _("user ID not found");
	ot = verify_get_key_ownertrust (GPGME_VALIDITY_UNDEFINED, NULL);
    }
    else {
	uid = pk.ext->uids->uid;
	ot = verify_get_key_ownertrust (pk.ctx->owner_trust, NULL);
    }
    
    if (!get_locale_timedate (sig->timestamp, made, DIM (made)-1))
	_snprintf (made, DIM (made)-1, "'unknown time'");
    if (!get_locale_timedate (time (NULL), ver, DIM (ver)-1))
	_snprintf (ver, DIM (ver)-1, "'unknown time'");
    stat = get_gpg_sigstat (sig->summary);
    p = new char[strlen (head) + strlen (stat) + strlen (made) + 
		 strlen (sig->fpr) + strlen (ot) + strlen (ver) +
		 strlen (uid) + 1];
    if (!p)
	BUG (0);
    if (strlen (sig->fpr) == 32) { /* RSA:MD5 */
	if (pk.ext != NULL)
	    keyid = pk.ext->key->subkeys->keyid+8;
	else
	    keyid = sig->fpr; /* show the fingerprint in case of problems. */
    }
    else
	keyid = get_keyid_from_fpr (sig->fpr);
    sprintf (p, head, made, ver, ot, stat, uid, keyid);
    *r_header = p;
    *r_footer = m_strdup (foot);
}


/* Show a human readable description of the given signature @sig. */
void
verify_show_signature_state (gpgme_signature_t sig)
{
    winpt_key_s key;
    const char *keyid, *uid;
    const char *s;
    char *pka_info = NULL, timebuf[128];
    int novalid = 0;

    assert (sig->fpr != NULL);
	
    memset (&key, 0, sizeof (key));
    if (!winpt_get_pubkey (sig->fpr, &key)) {
	s = verify_get_key_ownertrust (key.ctx->owner_trust, &novalid);
	uid = key.ext->uids->uid;	
    }
    else {
	s = "";
	uid = _("user ID not found");
    }

    if (!get_locale_timedate (sig->timestamp, timebuf, DIM (timebuf)-1))
	_snprintf (timebuf, DIM (timebuf)-1, "'unknown time'");

    if (strlen (sig->fpr) == 32) {
	if (key.ctx != NULL)
	    keyid = key.ctx->subkeys->keyid+8;
	else
	    keyid = sig->fpr; /* in case of problems show the fingerprint. */
    }	
    else
	keyid = get_keyid_from_fpr (sig->fpr);
    
    pka_info = get_pka_status (sig);
    log_box (_("Decrypt Verify"), novalid? MB_WARN : MB_OK, 
	     _("%s\n"
	       "%s\n"
               "Signature made: %s\n"
               "From \"%s\" using key ID 0x%s"
	       "%s %s\n%s"),
	       s, get_gpg_sigstat (sig->summary),		   
	       timebuf, uid, keyid,
	       novalid? "\nPrimary key fingerprint: " : "",
	       novalid? get_key_fpr (key.ctx) : "",
	       pka_info? pka_info : ""
	       );
    free_if_alloc (pka_info);
    winpt_release_pubkey (&key);
}


/* Convenient function to provide clipboard decryption.
   @hwnd is the parent window used for showing messsages.
   Return value: 0 on success. */
gpgme_error_t
clip_decrypt_dlg (HWND hwnd, int use_viewer)
{
    gpgme_error_t err;
    gpgme_ctx_t ctx = NULL;
    gpgme_decrypt_result_t res;
    gpgme_verify_result_t sigres;
    passphrase_cb_s pwd;
    int pgp_type = 0;

    /* allow to verify data generated by 'gpg -a --sign foo' */
    if (fm_assume_onepass_sig (NULL) == 1) {
	dialog_box_param (glob_hinst, (LPCSTR)IDD_WINPT_VERIFY, hwnd,
                          clip_verify_dlg_proc, 0, 
                          _("Verify"), IDS_WINPT_VERIFY);
	return 0;
    }

    err = gpgme_new (&ctx);
    if (err)
	BUG (NULL);
    set_gpg_passphrase_cb (&pwd, ctx, GPG_CMD_DECRYPT, hwnd, _("Decryption"));
    gpg_get_recipients (NULL, &pwd.recipients);

    err = gpgme_op_clip_decrypt (ctx);
    if (pwd.cancel)
	goto leave;
    if (gpgme_err_code (err) ==  GPG_ERR_BAD_PASSPHRASE)
	agent_del_cache (pwd.keyid);

    res = gpgme_op_decrypt_result (ctx);
    if (err && res->recipients && !is_seckey_available (res->recipients)) {
	gpgme_recipient_t r = res->recipients;
	char *u = get_key_userid (r->keyid+8);
	log_box (_("Decryption"), MB_ERR, 
		 _("Encrypted with %s key, ID 0x%s.%s\n"
		   "Decryption failed: secret key not available."), 
		   get_key_pubalgo (r->pubkey_algo), r->keyid+8, u);
	free_if_alloc (u);
	goto leave;
    }
    else if (res->unsupported_algorithm) {
	log_box (_("Decryption"), MB_ERR, _("Unsupported algorithm: %s"), 
		 res->unsupported_algorithm);
    }
    else if (err) {
	gpg_clip_get_pgptype (&pgp_type);
	if (gpgme_err_code (err) == GPG_ERR_NO_DATA && (pgp_type & PGP_MESSAGE))
	    msg_box (hwnd, _("Broken OpenPGP message (maybe: quoted printable "
			     "character in armor)."), _("Decryption"), MB_INFO);
	else
	    msg_box (hwnd, gpgme_strerror (err), _("Decryption"), MB_ERR);
	goto leave;
    }

    show_msg (GetDesktopWindow (), 1500, _("GnuPG Status: Finished"));

    sigres = gpgme_op_verify_result (ctx);
    if (sigres && sigres->signatures) {
	if (!use_viewer)
	    verify_show_signature_state (sigres->signatures);
	else
	    DialogBoxParam (glob_hinst, (LPCTSTR)IDD_WINPT_CLIPEDIT,
			    hwnd, clip_edit_dlg_proc, 
			    (LPARAM)sigres->signatures);
    }
    else if (use_viewer)
	DialogBoxParam (glob_hinst, (LPCTSTR)IDD_WINPT_CLIPEDIT,
			    hwnd, clip_edit_dlg_proc, 0);

leave:
    release_gpg_passphrase_cb (&pwd);
    gpgme_release (ctx);
    return err;
}
