/* wptGPGPrefsDlg.cpp - GnuPG Preferences
 *	Copyright (C) 2001-2006, 2007 Timo Schulz
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WinPT; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <windows.h>

#include "resource.h"
#include "wptNLS.h"
#include "wptTypes.h"
#include "wptErrors.h"
#include "wptGPG.h"
#include "wptRegistry.h"
#include "wptCommonCtl.h"
#include "wptW32API.h"


/* Enable all edit items if @val is TRUE, disable them otherwise. */
static void
activate_items (HWND dlg, BOOL val)
{
    EnableWindow (GetDlgItem (dlg, IDC_GPGPREFS_EXEDIR), val);
    EnableWindow (GetDlgItem (dlg, IDC_GPGREFS_EXEDLG), val);
    EnableWindow (GetDlgItem (dlg, IDC_GPGPREFS_LOCALE), val);
    EnableWindow (GetDlgItem (dlg, IDC_GPGPREFS_LOCDLG), val);
    EnableWindow (GetDlgItem (dlg, IDC_GPGPREFS_HOMEDIR), val);
    EnableWindow (GetDlgItem (dlg, IDC_GPGPREFS_HOMEDLG), val);
}


/* Load the GPG4WIN default values and disabled the
   dialog items to indicate these are fixed values.
   Return value: true if GPG4win was found. */
static bool
load_gpg4win_values (HWND dlg)
{
    char *path;
    char *p;

    path = get_reg_entry_gpg4win (NULL);
    if (!path)
	return false;
    p = make_filename (path, "gpg", "exe");
    if (p) {
	if (file_exist_check (p) == 0) {
	    SetDlgItemText (dlg, IDC_GPGPREFS_EXEDIR, p);
	    EnableWindow (GetDlgItem (dlg, IDC_GPGPREFS_EXEDIR), FALSE);
	    EnableWindow (GetDlgItem (dlg, IDC_GPGREFS_EXEDLG), FALSE);
	}
	free_if_alloc (p);
    }
    free_if_alloc (path);

    path = get_reg_entry_gpg ("HomeDir");
    p = multi_gnupg_path (1);
    if (path && dir_exist_check (path) == 0 && p && strcmp (path, p)) {
	/* The 'HomeDir' has a higher priority so if the key exists and is 
	   different from the multi user path, we force the use of it. */
	free_if_alloc (p);
	p = path;
    }
    else
	free_if_alloc (path);
    if (p) {
	SetDlgItemText (dlg, IDC_GPGPREFS_HOMEDIR, p);
	EnableWindow (GetDlgItem (dlg, IDC_GPGPREFS_HOMEDIR), FALSE);
	EnableWindow (GetDlgItem (dlg, IDC_GPGPREFS_HOMEDLG), FALSE);
	free_if_alloc (p);
    }
    return true;
}


/* Load the GPG values direct from the registry. */
static void
load_registry_values (HWND dlg)
{
    char *p;

    p = get_reg_entry_gpg ("HomeDir");
    if (p) {
	SetDlgItemText (dlg, IDC_GPGPREFS_HOMEDIR, p);
	free_if_alloc (p);
    }
    p = get_reg_entry_gpg ("gpgProgram");
    if (p) {
	SetDlgItemText (dlg, IDC_GPGPREFS_EXEDIR, p);
	free_if_alloc (p);
    }

    /* if there is no 'HomeDir' registry value but an existing
       GPG multipath, we use this as the home dir instead. */
    if (!item_get_text_length (dlg, IDC_GPGPREFS_HOMEDIR)) {
	p = multi_gnupg_path (0);
	if (p && dir_exist_check (p) == 0)
	    SetDlgItemText (dlg, IDC_GPGPREFS_HOMEDIR, p);
	free_if_alloc (p);
    }
}


/* Dialog box procedure for the GPG preferences. */
BOOL CALLBACK
gpgprefs_dlg_proc (HWND dlg, UINT msg, WPARAM wparam, LPARAM lparam)
{
    static int gpg4win = false;
    char exedir[512];
    char homedir[512];
    char *p = NULL, t[256];
    const char *s;
    const char *folder;
    config_file_t opt = NULL;
    conf_option_t e;
    UINT n;
    
    switch (msg) {
    case WM_INITDIALOG:
        SetWindowText (dlg, _("GnuPG Preferences"));
        SetDlgItemText (dlg, IDC_GPGPREFS_HOMEINF,
                       _("GnuPG home directory  (where both keyrings are located)"));
        SetDlgItemText ( dlg, IDC_GPGPREFS_EXEINF,
                        _("GnuPG exe file location (full path with added gpg.exe)"));
	SetDlgItemText (dlg, IDC_GPGPREFS_ASKLEVEL, _("Ask for the signature class during key sign"));
	SetDlgItemText (dlg, IDC_GPGPREFS_ASKEXPIRE, _("Allow to set an expiration date for signatures"));
	SetDlgItemText (dlg, IDC_GPGPREFS_CMTINF, _("Comment in armored files"));
	SetDlgItemText (dlg, IDC_GPGPREFS_ENCINF, _("Encrypt to this key"));
	SetDlgItemText (dlg, IDC_GPGPREFS_ALLOPTINF, _("General GPG options"));
	SetDlgItemText (dlg, IDC_GPGPREFS_HOMEDLG, _("Browse..."));
	SetDlgItemText (dlg, IDC_GPGREFS_EXEDLG, _("Browse..."));
	SetDlgItemText (dlg, IDC_GPGPREFS_OVRDEFAULT, _("&Overwrite default settings"));
	SetDlgItemText (dlg, IDCANCEL, _("&Cancel"));

	gpg4win = load_gpg4win_values (dlg);
	if (!gpg4win) {
	    load_registry_values (dlg);
	    CheckDlgButton (dlg, IDC_GPGPREFS_OVRDEFAULT, BST_CHECKED);
	}

	p = get_gnupg_cfgfile ();
	if (p) {
	    parse_config (p, &opt);
	    free_if_alloc (p);
	    if (opt) {
		if (conf_find_option (opt, "ask-cert-expire"))
		    CheckDlgButton (dlg, IDC_GPGPREFS_ASKEXPIRE, BST_CHECKED);
		if (conf_find_option (opt, "ask-cert-level"))
		    CheckDlgButton (dlg, IDC_GPGPREFS_ASKLEVEL, BST_CHECKED);
		e = conf_find_option (opt, "comment");
		if (e != NULL)
		    SetDlgItemText (dlg, IDC_GPGPREFS_COMMENT, e->val);
		e = conf_find_option (opt, "encrypt-to");
		if (e != NULL)
		    SetDlgItemText (dlg, IDC_GPGPREFS_ENCTO, e->val);
		release_config (opt);
	    }
	}

#ifdef WINPT_MOBILE
	/* In mobile mode we do not allow to change the default settings
	   because this would not make any sense. */
	CheckDlgButton (dlg, IDC_GPGPREFS_OVRDEFAULT, BST_UNCHECKED);
	EnableWindow (GetDlgItem (dlg, IDC_GPGPREFS_OVRDEFAULT), FALSE);
	activate_items (dlg, FALSE);
#endif

	center_window (dlg, NULL);
        SetForegroundWindow (dlg);
        return TRUE;

    case WM_DESTROY:
	gpg4win = false;
	break;
        
    case WM_COMMAND:
	if (HIWORD (wparam) == BN_CLICKED &&
	    LOWORD (wparam) == IDC_GPGPREFS_OVRDEFAULT) {
	    if (IsDlgButtonChecked (dlg, LOWORD (wparam))) {
		activate_items (dlg, TRUE);
		gpg4win = false;
	    }
	    else {
		activate_items (dlg, FALSE);
		gpg4win = true;
	    }
	    break;
	}

        switch (LOWORD (wparam)) {
        case IDC_GPGPREFS_SAVE:
            if (!GetDlgItemText (dlg, IDC_GPGPREFS_HOMEDIR, 
				 homedir, DIM (homedir) -1)) {
                msg_box (dlg, _("Please enter the GnuPG home directory."),
			 _("Preferences"), MB_ERR);
                return FALSE;
            }
            if (dir_exist_check (homedir)) {
		_snprintf (t, DIM (t) - 1, "%s: %s", homedir,
			   winpt_strerror (WPTERR_DIR_OPEN));
                msg_box (dlg, t, _("Preferences"), MB_ERR);
                return FALSE;
            }
            
            if (!gpg4win && set_reg_entry_gpg ("HomeDir", homedir)) {
                msg_box (dlg, _("Could not save 'HomeDir' in the registry."), 
			 _("Preferences"), MB_ERR);
                return FALSE;
            }
            if (!GetDlgItemText (dlg, IDC_GPGPREFS_EXEDIR,
				 exedir, DIM (exedir) -1)) {
                msg_box (dlg, _("Please enter where GPG.exe is located."),
			 _("Preferences"), MB_ERR);
                return FALSE;
            }
            if (file_exist_check (exedir)) {
                msg_box (dlg, _("Could not find the GPG program in this directory."),
			 _("Preferences"), MB_ERR);
                return FALSE;
            }
            if (!gpg4win && set_reg_entry_gpg ("gpgProgram", exedir)) {
                msg_box (dlg, _("Could not save 'gpgProgram' in the registry"),
			 _("Preferences"), MB_ERR);
                return FALSE;
            }
            
	    p = get_gnupg_cfgfile ();
	    if (!p) {
		msg_box (dlg, _("Could not get GPG config file"), 
			 _("Preferences"), MB_ERR);
		EndDialog (dlg, FALSE);
		return TRUE;
	    }
	    parse_config (p, &opt);

	    if (IsDlgButtonChecked (dlg, IDC_GPGPREFS_ASKLEVEL)) {
		conf_modify_entry (opt, ENTRY_SINGLE, "ask-cert-level", NULL);
		reg_prefs.gpg.ask_cert_level = 1;
	    }
	    else {
		conf_delete_option (opt, "ask-cert-level");
		reg_prefs.gpg.ask_cert_level = 0;
	    }
	    if (IsDlgButtonChecked (dlg, IDC_GPGPREFS_ASKEXPIRE)) {
		conf_modify_entry (opt, ENTRY_SINGLE, "ask-cert-expire", NULL);
		reg_prefs.gpg.ask_cert_expire = 1;
	    }
	    else {
		conf_delete_option (opt, "ask-cert-expire");
		reg_prefs.gpg.ask_cert_expire = 0;
	    }

	    n = GetDlgItemText(dlg, IDC_GPGPREFS_COMMENT, t, DIM (t)-1);
            if (n > 0)
		conf_modify_entry (opt, ENTRY_MULTI, "comment", t);
	    else if (n == 0)
		conf_modify_entry (opt, ENTRY_MULTI, "comment", "\"\"");
	    else
		conf_delete_option (opt, "comment");

	    n = GetDlgItemText (dlg, IDC_GPGPREFS_ENCTO, t, DIM (t)-1);
            if (n > 0)
		conf_modify_entry (opt, ENTRY_MULTI, "encrypt-to", t);
            else
		conf_delete_option (opt, "encrypt-to");

	    commit_config (p, opt);
	    release_config (opt);
	    /* only return TRUE if the home dir has been changed. */
	    if (SendDlgItemMessage (dlg, IDC_GPGPREFS_HOMEDIR, EM_GETMODIFY, 0, 0)) {
		set_gnupg_default_key (NULL);
		EndDialog (dlg, TRUE);		
	    }
	    else
		EndDialog (dlg, FALSE);
            return TRUE;
            
	case IDC_GPGPREFS_HOMEDLG:
	    folder = get_folder_dlg (dlg, _("Choose GPG Home Directory"), NULL);
	    if (folder) {
		char *name;

		SetDlgItemText (dlg, IDC_GPGPREFS_HOMEDIR, folder);
		SendDlgItemMessage (dlg, IDC_GPGPREFS_HOMEDIR,
				    EM_SETMODIFY, (WPARAM)(UINT)TRUE, 0);
		if (GetDlgItemText (dlg, IDC_GPGPREFS_EXEDIR,
				    exedir, DIM (exedir)-1) > 0)
		    break;
		name = make_filename (folder, "gpg", "exe");
		if (file_exist_check (name) == 0)
		    SetDlgItemText (dlg, IDC_GPGPREFS_EXEDIR, name);
		free_if_alloc (name);
	    }
	    return TRUE;
	    
        case IDC_GPGREFS_EXEDLG:
            // TODO: use gpgme_ctx_set_engine_info instead.
	    n = msg_box (dlg, _("Please be aware that the modifcation of "
				"the value will affect all installed "
				"programs which use GPG via the "
				"GPGME interface library\n\n"
				"Do you really want to continue?"),
			 _("GPG WARNING"), MB_WARN_ASK);
	    if (n != IDYES)
		return TRUE;
	    
	    s = get_fileopen_dlg (dlg, _("Choose GPG Binary"), 
				  "Executable Files (*.exe)\0*.exe\0\0",
				  NULL);
            if (s)
                SetDlgItemText (dlg, IDC_GPGPREFS_EXEDIR, s);
            return TRUE;
	    
	case IDCANCEL:
	    EndDialog (dlg, FALSE);
	    return TRUE;
        }
        break;
    }
    
    return FALSE;
}
