/* wptProxySettingsDlg.cpp - Dialog for the proxy settings
 *	Copyright (C) 2002-2006 Timo Schulz
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <windows.h>
#include <ctype.h>
#include <stdio.h>

#include "resource.h"
#include "wptTypes.h"
#include "wptGPG.h"
#include "wptKeyserver.h"
#include "wptW32API.h"
#include "wptErrors.h"
#include "wptNLS.h"
#include "wptRegistry.h"

// TODO: there are segfault problems in the password mode

#define combo_get_sel(dlg, id) \
    SendDlgItemMessage ((dlg), (id), CB_GETCURSEL, 0, 0)


static int
set_reg_entry_keyserver_int (const char *key, int val)
{
    char buf[32];

    _snprintf (buf, DIM (buf)-1, "%d", val);
    return set_reg_entry_keyserver (key, buf);
}


/* Save the proxy settings in the registry. */
int
set_reg_proxy_prefs (keyserver_proxy_t prox)
{    
    int rc;
    
    rc = set_reg_entry_keyserver_int ("Proto", prox->proto);
    if (!rc)
	rc = set_reg_entry_keyserver ("Host", prox->host? prox->host : "");
    if (!rc)
	rc = set_reg_entry_keyserver_int ("Port", prox->port);
    if (!rc)
	rc = set_reg_entry_keyserver ("User", prox->user? prox->user : "");
    if (!rc)
	rc = set_reg_entry_keyserver ("Pass", prox->pass? prox->pass : "");
    return rc;
}


/* Check if the edit control (@id) contains a valid number. */
static int
check_number (HWND dlg, int id)
{
    char buf[32];
    size_t i;

    i = GetDlgItemText (dlg, id, buf, DIM (buf)-1);
    if (!i)
	return -1;
    for (i = 0; i < strlen (buf); i++) {
	if (!isdigit (buf[i]))
	    return -1;
    }
    return 0;
}


static int
read_proxy (HWND dlg, keyserver_proxy_t ctx)
{
    char t[512];
    int ncount;

    ncount = GetDlgItemText (dlg, IDC_PROXY_PWD, t, DIM (t)-1);
    if (ncount > 0)
        ctx->pass = m_strdup (t);
    ncount = GetDlgItemText (dlg, IDC_PROXY_USER, t, DIM (t)-1);
    if (ncount > 0)
        ctx->user = m_strdup (t);
    ncount = GetDlgItemText (dlg, IDC_PROXY_HOST, t, DIM (t)-1);
    if (ncount > 0) {
	int pos = 0;

	if (check_inet_address (t)) {
	    msg_box (dlg, _("Invalid host or IP address."), _("Proxy Settings"), MB_ERR);
	    return -1;
	}
	/* TODO: check prefix */
	if (!strnicmp (t, "http://", 7))
	    pos = 7;
        ctx->host = m_strdup (t + pos);
    }
    else {
	msg_box (dlg, _("Please enter the proxy hostname."), 
			 _("Proxy Settings"), MB_ERR);
	return -1;
    }

    if (check_number (dlg, IDC_PROXY_PORT)) {
	msg_box (dlg, _("Invalid port number."), _("Proxy Settings"), MB_ERR);
	return -1;
    }

    ctx->port = GetDlgItemInt (dlg, IDC_PROXY_PORT, NULL, FALSE);
    if (ctx->port < 0 || ctx->port > 65535) {
	msg_box (dlg, _("Please select a value from 0-65535 for the port"), 
		_("Proxy Settings"), MB_INFO);
	return -1;
    }
    if (ctx->user && !ctx->pass || !ctx->user && ctx->pass) {
	msg_box (dlg, _("When you want to use authentication, "
		        "please fill out both fields."), _("Proxy Settings"), MB_ERR);
	return -1;
    }
    if (ctx->host && ctx->port==0 || !ctx->host && ctx->port != 0) {
	msg_box (dlg, _("Please enter a host name and a port."),
		 _("Proxy Settings"), MB_INFO);
	return -1;
    }
    ctx->proto = combo_get_sel (dlg, IDC_PROXY_PROTO);
    return 0;
}


/* Enable or disable the authentication elements based on
   the value @val. */
static void
enable_proxy_auth (HWND dlg, int val)
{
    int mode = val? TRUE : FALSE;

    EnableWindow (GetDlgItem (dlg, IDC_PROXY_PWD), mode);
    EnableWindow (GetDlgItem (dlg, IDC_PROXY_USER), mode);
}


/* Fill in all valid proxy protocols. */
static void
init_proxy_protocols (HWND dlg)
{
    HWND cb = GetDlgItem (dlg, IDC_PROXY_PROTO);

    combox_add_string (cb, (char *)"NONE");
    combox_add_string (cb, (char *)"HTTP");
    // TODO: combox_add_string (cb, (char *)"SOCKS5");
    SendMessage (cb, CB_SETCURSEL, (WPARAM)0, 0);
}


/* Dialog box procedure for proxy authentication. */
BOOL CALLBACK
keyserver_proxy_dlg_proc (HWND dlg, UINT msg, WPARAM wparam, LPARAM lparam)
{   
    int auth = 0;

    switch (msg) {
    case WM_INITDIALOG:
        if (proxy.host != NULL) {
            SetDlgItemText (dlg, IDC_PROXY_HOST, proxy.host);
            SetDlgItemInt( dlg, IDC_PROXY_PORT, proxy.port, FALSE );
            if (proxy.user != NULL) {
                SetDlgItemText (dlg, IDC_PROXY_USER, proxy.user);
		CheckDlgButton (dlg, IDC_PROXY_AUTH, BST_CHECKED);
		auth++;
	    }
	    if (proxy.pass != NULL) {
                SetDlgItemText (dlg, IDC_PROXY_PWD, proxy.pass);
		auth++;
	    }
        }
	enable_proxy_auth (dlg, auth);
	SetDlgItemText (dlg, IDC_PROXY_HOSTINF, _("Proxy host name or IP address"));
	SetDlgItemText (dlg, IDC_PROXY_AUTH, _("Server requires &authentication"));
	SetDlgItemText (dlg, IDC_PROXY_USERINF, _("User name"));
	SetDlgItemText (dlg, IDC_PROXY_PWDINF, _("Password"));
	SetDlgItemText (dlg, IDC_PROXY_PROTOINF, _("Proxy type"));
	SetDlgItemText (dlg, IDC_PROXY_AUTHINF, _("Authentication"));
	SetWindowText (dlg, _("Proxy Settings"));
	init_proxy_protocols (dlg);
	SendDlgItemMessage (dlg, IDC_PROXY_PROTO, CB_SETCURSEL, 
			    (WPARAM)proxy.proto, 0);
	center_window (dlg, NULL);
	SetForegroundWindow (dlg);
        break;
        
    case WM_COMMAND:
	switch (HIWORD (wparam)) {
	case CBN_SELCHANGE:
	    if (combo_get_sel (dlg, (int)LOWORD (wparam))) {
	    }
	    else {
	    }	
	    break;

        case BN_CLICKED:
	    switch ((int)LOWORD (wparam)) {
	    case IDC_PROXY_AUTH:
		if (IsDlgButtonChecked (dlg, IDC_PROXY_AUTH))
		    enable_proxy_auth (dlg, 1);
		else
		    enable_proxy_auth (dlg, 0);
		break;
	    }
	    break;
	}

        switch (LOWORD (wparam)) {
        case IDOK:
	    if (IsDlgButtonChecked (dlg, IDC_PROXY_AUTH) &&
		item_get_text_length (dlg, IDC_PROXY_USER) == 0 &&
		item_get_text_length (dlg, IDC_PROXY_PWD) == 0) {
		msg_box (dlg, _("Please fill out all required fields to enable proxy authentication."),
			 _("Proxy Settings"), MB_WARN);
		return TRUE;
	    }
	    kserver_proxy_release (&proxy);
	    if (combo_get_sel (dlg, IDC_PROXY_PROTO) != PROXY_PROTO_NONE) {
		if (read_proxy (dlg, &proxy)) 
		    return FALSE;
	    }
	    set_reg_proxy_prefs (&proxy);
            EndDialog (dlg, TRUE);
            return TRUE;
            
        case IDCANCEL:
            EndDialog (dlg, FALSE);
            return FALSE;
        }
        break;
    }
    
    return FALSE;
}
